<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Base controller for all controllers in application
 */
class BaseController extends CController
{
    private $_assetsPath;
    private $_clientParams;

    public $layout = '//layouts/gportal';
    public $mailLayout = '//layouts/mail';

    protected function beforeAction($action) {
        $this->layout = Yii::app()->getModule('settings')->layoutForBackend;
        return parent::beforeAction($action);
    }
    /**
     * Temporary dummy method
     *
     * @return object with configuration info
     */
    public function getClient()
    {
        // parse domain
        // get configuration for this domain
        // return configuration object for domain
        $this->_clientParams = new stdClass;
        $this->_clientParams->username = Yii::app()->params['client']['username'];
        $this->_clientParams->contentPath = Yii::app()->params['storageFolder'] . DIRECTORY_SEPARATOR . $this->_clientParams->username;
        return $this->_clientParams;
    }

    /**
     * CAssetManager is a Web application component that manages private files
     * (called assets) and makes them accessible by Web clients.
     * It achieves this goal by copying assets to a Web-accessible directory
     * and returns the corresponding URL for accessing them.
     *
     * @throws CException
     * @return string the root directory storing the published asset files. Defaults to 'WebRoot/assets'.
     */
    public function getAssetsPath()
    {
        if (is_null($this->_assetsPath)) {
            $assetsPath = ($this->module !== null) ? $this->module->name . '/assets' : Yii::app()->controller->client->contentPath;
            if (($assetsPath = realpath($assetsPath)) === false || !is_dir($assetsPath) || !is_writable($assetsPath)) {
                throw new CException(
                    Yii::t('yii', 'CAssetManager.basePath "{path}" is invalid. Please make sure the directory exists and is writable by the Web server process.', array('{path}' => $assetsPath))
                );
            }
            $this->_assetsPath = Yii::app()->assetManager->publish($assetsPath, false, -1, false);
        }
        return $this->_assetsPath;
    }

    /**
     * Overridden parent method. Add possibility to load view files from /custom folder
     *
     * @param string $viewName name of view
     *
     * @return bool|string
     */
    public function getViewFile($viewName)
    {
        $viewFile = parent::getViewFile($viewName);

        if (isset(Yii::app()->params['customViews']) && Yii::app()->params['customViews'] == true) {
            $customSection = realpath(Yii::getPathOfAlias('site.custom'));
            $commonSection = realpath(Yii::getPathOfAlias('site.common'));
            $frontendSection = realpath(Yii::getPathOfAlias('site.frontend'));
            $customView = str_replace(array($commonSection, $frontendSection), $customSection, $viewFile);
            if (file_exists($customView)) {
                return $customView;
            }
        }

        return $viewFile;
    }

    function getJs($path){
        echo "<script src=".$path."></script>";
    }

    public function renderMail($view, $title = null, $data, $return = true)
    {
        $pathToView = Yii::getPathOfAlias($view);
        $module = null;
        preg_match("/\\\\modules\\\\(?P<module_id>\w+)\\\\/", $pathToView, $matches);
        if(!empty($matches)){
            $module = $matches['module_id'];
            $module = Yii::app()->getModule($module);
            $this->mailLayout = isset($module->mainMailTemplate)?$module->mainMailTemplate:$this->mailLayout;
        }
        $output = $this->renderPartial($view, $data, true);
        if (($layoutFile = $this->getLayoutFile($this->mailLayout)) !== false)
            $output = $this->renderFile($layoutFile, array('content' => $output, 'title' => $title), true);

        if ($return)
            return $output;
        else
            echo $output;
    }

    public function redirect($url,$terminate=true,$statusCode=302)
    {
        //escaping from CRLF injection
        $exceptModules = array('user');
        $moduleId = isset(Yii::app()->controller->module->id)?Yii::app()->controller->module->id:false;
        if(!$moduleId || ($moduleId && !in_array($moduleId, $exceptModules))){
            new AppHelper(); //this file contains function http_build_url
            $url = http_build_url('', parse_url($url));
        }
        parent::redirect($url, $terminate, $statusCode);
    }
    /**
     * Overridden parent method. Add possibility to load widget classes from /custom folder
     *
     * @param string $className class name (can be in path alias format)
     * @param array $properties initial property values
     * @return CWidget the fully initialized widget instance.
     */
    public function createWidget($className,$properties=array())
    {
        if (isset(Yii::app()->params['customWidgets']) && Yii::app()->params['customWidgets'] == true) {

            $pattern = '/^(.*?modules\.)?(.*)$/';
            $replacement = 'custom.modules.${2}';
            $custClassName = preg_replace($pattern, $replacement, $className);

            if (file_exists(Yii::getPathOfAlias($custClassName).'.php')) {
                $className = $custClassName;
            }
        }
        return parent::createWidget($className,$properties);
    }
}
