<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Class for helping in work with headers
 */
class HeadersHelper
{

    /**
     * Build content headers for file content.
     *
     * @param string $path    file path
     * @param string $content file content
     *
     * @return array headers
     */
    public static function forFile($path, $content = null)
    {
        $headers = array();
        $headers[] = "Content-Type: " . CFileHelper::getMimeTypeByExtension($path);
        if ($content) {
            $headers[] = "Content-Length: " . mb_strlen($content);
	    $contentPath = Yii::app()->getModule('contentSource')->getContentSource()->contentFile($path);
	    $headers[] = "Last-Modified: " . gmdate("D, d M Y H:i:s", filemtime($contentPath)) . " GMT";
        }
        return $headers;
    }

    /**
     * Build expires headers for file
     *
     * @param int $seconds seconds before content expires
     *
     * @return array headers
     */
    public static function expires($seconds)
    {
        $headers = array();
        $headers[] = "Pragma: public";
        $headers[] = "Expires: " . date("r", time() + $seconds);

        return $headers;
    }
    
    /**
     * 
     * Build cache control header for file
     * 
     * @param string $filename
     * @param int $seconds how long the file should be cached in browser
     * @return array headers
     */
    public static function cache($seconds = null)
    {
    	$headers = array();
    	
    	$cache_string = "Cache-Control: public"; 
    	
    	if(isset($seconds) && is_numeric($seconds) && $seconds > 0) {
    		$cache_string = "Cache-Control: max-age=".$seconds.", public";
    	}
    	
    	$headers[] = $cache_string;
    	
    	return $headers;
    }
}