<?php
/**
 * This command processes one channel with name $channel ,
 * and if $clear = 1 truncate tables before fill them
 *
 * go to gportal/common:
 * yiic processAPA --channel=test --clear=1
 *
 * Install incron :
 * 1)If Debian / Ubuntu Linux:
 * sudo apt-get install incron
 *
 * 2)If RHEL / Fedora / CentOS Linux:
 * sudo yum install incron
 *
 * Start/Stop
 * sudo /etc/init.d/incron stop
 * sudo /etc/init.d/incron start
 *
 * More info here:
 * http://www.cyberciti.biz/faq/linux-inotify-examples-to-replicate-directories/
 *
 * @author Andrew Voloshin <andrew.voloshin@oberig.com>
 *
 */
class processAPACommand extends CConsoleCommand
{
    /**
     * default action
     * it processes channel with name $channel
     *
     * @param string $channel name of channel
     * @param int $clear whether to truncate tables before import
     */
    public function actionIndex($channel = false,$clear = false) {

        Yii::app()->getModule('apalines');

        if ($clear) {
            if ($channel) {
                $channels = array($channel);
            } else {
                $channels = Yii::app()->getModule('apalines')->channels;
                $channels = array_keys($channels);
            }
            $notExistFolders = array();
            foreach ($channels as $name) {
                $ch = new Channel($name);
                if (!file_exists($ch->path)) {
                    $notExistFolders[] = $name;
                    continue;
                }
                $news = ApalinesNews::model()->findAll('deleted <> 1');
                foreach ($news as $line) {
                    $line->delete();
                }
            }
            $channels = implode(', ', $channels);
            echo "News of the channels: {$channels} are removed\n";
            if (!empty($notExistFolders)) {
                $notExistFolders = implode(', ', $notExistFolders);
                echo "Channel folders of: {$notExistFolders} are not exist\n";
            }
            Yii::app()->end();
        }

        if (!$channel) {
            echo "Channel name is not passed";
            Yii::app()->end();
        }

        $channel = new Channel($channel);

        if (!$channel->isNew()) {
            echo "Fertig.txt does not exist in the channel folder\n";
            Yii::app()->end();
        }
        $channel->clearOld();

        $channel->moveToWork();
        $channel->processWork();
    }
    /**
     * action sets up incrontab to execute import command with current module settings
     */
    public function actionSetup() {

        $incronPath = "/var/spool/incron/root";
        $module = Yii::app()->getModule('apalines');
        $channels = $module->channels;
        $incronContent = '';
        $yiicPath = Yii::getPathOfAlias('site.common').DIRECTORY_SEPARATOR."yiic";
        $events = "IN_CREATE,IN_CLOSE_WRITE,IN_MOVED_TO";

        foreach($channels as $alias => $channel){

            $path = Yii::getPathOfAlias($channel['delivery_path']);
            $command = "$yiicPath processApa index --channel=$alias";
            $incronContent .= "$path $events $command\n";
        }

        file_put_contents($incronPath, $incronContent);
    }
}