<?php
/**
 * Created by JetBrains PhpStorm.
 * User: Andrew
 * Date: 07.06.13
 * Time: 19:35
 * To change this template use File | Settings | File Templates.
 */

/**
 * Class for process a channel (import news from xml into database)
 *
 * @author Andrew Voloshin <andrew.voloshin@oberig.com>
 *
 * all parameters will be described in the part of current channel
 * because 1 object of this class = 1 channel !!!
 *
 */

class Channel
{
    /**
     * @var string name of the channel
     */
    public $name;
    /**
     * @var string path to fertig file of the channel
     */
    public $fertigPath;
    /**
     * @var string path to main folder of the channel
     */
    public $path;
    /**
     * @var string path to work folder of the channel
     */
    public $workPath;
    /**
     * @var string path to done folder of the channel
     */
    public $donePath;
    /**
     * @var string path to error folder of the channel
     */
    public $errorPath;
    /**
     * @var string path to public folder (where placed images and videos folders) of the channel
     */
    public $wwwPath;
    /**
     * @var string path to images files
     */
    public $imagesPath;
    /**
     * @var string path to images files
     */
    public $videosPath;
    /**
     * @var array extensions of removal files
     */
    private $removalExt = array('del');
    /**
     * @var array extensions of news files
     */
    private $newsExt = array('xml');
    /**
     * @var array extensions of images files
     */
    private $imagesExt = array('jpg');
    /**
     * @var array extensions of video files
     */
    private $videosExt = array('mp4');
    /**
     * @var string log file name
     */
    private $log_filename;
    /**
     * @var array relation between news files
     */
    private $relations = array();
    /**
     * @var boolean whether delivery folder was updated
     */
    private $isNew = false;
    /**
     * @var string channel instance
     */
    public static $instance;
    /**
     * Constructor initializes all parameters and makes necessary folders if they are not
     */
    public function __construct($name)
    {
        $time = time();
        $this->log_filename .= "log_{$time}.txt";
        $this->name = $name;
        $channelAlias = "apalines.content.$name";

        $module = Yii::app()->getModule('apalines');
        if(isset($module->channels[$name]['delivery_path'])){
            $channelAlias = $module->channels[$name]['delivery_path'];
        }

        $this->path = Yii::getPathOfAlias($channelAlias);
        $this->fertigPath = $this->path.DIRECTORY_SEPARATOR.'fertig.txt';

        if (file_exists($this->fertigPath)) {
            unlink($this->fertigPath);
            $this->isNew = true;
        }

        //Log router
        if (file_exists($this->path)) {
            Yii::app()->log->routes['file']->setLogPath($this->path);
            Yii::app()->log->routes['file']->setLogFile($this->log_filename);
        }
        //Uncomment to overwrite log file
        /*
        if(file_exists($this->makePath($this->path,$this->log_filename))){
            unlink($this->makePath($this->path,$this->log_filename));
        }
        **/
        $this->workPath  = Yii::getPathOfAlias("$channelAlias.work");
        $this->donePath  = Yii::getPathOfAlias("$channelAlias.done");
        $this->errorPath = Yii::getPathOfAlias("$channelAlias.error");

        $wwwApaPath = Yii::getPathOfAlias("site.frontend.www.apalines");
        $this->wwwPath = Yii::getPathOfAlias("site.frontend.www.apalines.$name");
        $this->imagesPath = Yii::getPathOfAlias("site.frontend.www.apalines.$name.images");
        $this->videosPath = Yii::getPathOfAlias("site.frontend.www.apalines.$name.videos");

        if (file_exists($this->path)) {
            if (!file_exists($this->workPath)) {
                mkdir($this->workPath);
            }
            if (!file_exists($this->donePath)) {
                mkdir($this->donePath);
            }
            if (!file_exists($this->errorPath)) {
                mkdir($this->errorPath);
            }
            if (!file_exists($wwwApaPath)) {
                mkdir($wwwApaPath);
            }
            if (!file_exists($this->wwwPath)) {
                mkdir($this->wwwPath);
            }
            if (!file_exists($this->imagesPath)) {
                mkdir($this->imagesPath);
            }
            if (!file_exists($this->videosPath)) {
                mkdir($this->videosPath);
            }
        }

        self::$instance = $this;
    }
    /**
     * Function checks whether channel has updates
     *
     * @return boolean new or not
     */
    public function isNew()
    {
        return $this->isNew;
    }
    /**
     * Function returns extension of filename
     *
     * @param string $filename - name of file
     *
     * @return string - extension
     */
    function getExt($filename)
    {
        return substr(strrchr($filename, '.'), 1);
    }
    /**
     * Function processes files in work folder: distributes what handler should process each file
     */
    function processWork()
    {
        if (is_dir($this->workPath)) {
            if ($dh = opendir($this->workPath)) {
                while (($file = readdir($dh)) !== false) {
                    if (!is_file($this->makeWorkPath($file)) || $file == '.DS_Store') {
                        continue;
                    }
                    //Process deletions
                    if (in_array($this->getExt($file),$this->removalExt)){
                        $xml = simplexml_load_file($this->makeWorkPath($file));
                        $this->processRemoval($xml, $file);
                    }
                }
                //Step 2. Update meldung
                rewinddir($dh);
                while (($file = readdir($dh)) !== false) {
                    if (!is_file($this->makeWorkPath($file)) || $file == '.DS_Store') {
                        continue;
                    }
                    //Process news, images, videos
                    else if (in_array($this->getExt($file),$this->newsExt)) {
                        $xml = simplexml_load_file($this->makeWorkPath($file));
                        $this->processNews($xml, $file, array('DOK_MELDUNG'));
                    }
                }
                //Step 3. Update build
                rewinddir($dh);
                while (($file = readdir($dh)) !== false) {
                    if (!is_file($this->makeWorkPath($file)) || $file == '.DS_Store') {
                        continue;
                    }
                    //Process multimedia
                    else if (in_array($this->getExt($file),$this->newsExt)) {
                        $xml = simplexml_load_file($this->makeWorkPath($file));
                        $this->processNews($xml, $file, array('DOK_BILD', 'DOK_VIDEO'));
                    }
                    //Move images
                    else if (in_array($this->getExt($file),$this->imagesExt)) {
                        rename($this->makeWorkPath($file),
                            $this->makeImagesPath($file));
                    }
                    //Move videos
                    else if (in_array($this->getExt($file),$this->videosExt)) {
                        rename($this->makeWorkPath($file),
                            $this->makeVideosPath($file));
                    }
                }
                closedir($dh);
            }
        }
        //Make relations between imported news and multimedia
        $this->makeRelations();
        //Remove multimedia which have not parent news
        $this->checkMMParents();
    }
    /**
     * Function processes: news and multimedia(images/video) xml files
     *
     * @param object $xml SimpleXMLElement
     * @param string $xml_filename name of xml file
     * @param array $specific what files should be processed
     */
    function processNews($xml, $xml_filename = null, $specific = false)
    {
        $parserXML = new parserXML($xml);
        $isSuccess = false;
        $docType = $parserXML->getDocType();

        if ($specific && is_array($specific) && !in_array($docType, $specific)) {
            return;
        }

        switch($parserXML->getDocType()) {
            case 'DOK_MELDUNG':
                $isSuccess = $this->processMeldung($parserXML, $xml_filename);
                break;
            case 'DOK_BILD' || 'DOK_VIDEO':
                $isSuccess = $this->processBild($parserXML, $xml_filename);
                break;
        }
        if ($isSuccess) {
            //Moving to done folder
            $this->moveFileToDone($xml_filename);
        } else {
            //Moving to error folder
            $this->moveFileToError($xml_filename);
        }
    }
    /**
     * Function processes news xml files
     *
     * @param object $parserXML object of parserXml class
     * @param string $xml_filename name of xml file
     *
     * @return boolean whether import finished successfully
     */
    function processMeldung($parserXML, $xml_filename = null)
    {
        $line = ApalinesNews::model()->find('docid = :docid',array(':docid'=>$parserXML->getDocId()));
        $isExist = isset($line);

        if (!isset($line)) {
            $line = new ApalinesNews();
            $line->docid = $parserXML->getDocId();
            $line->channel = $this->name;
        } else {
            $line->deleteFiles();
        }

        $line->apa_id = $parserXML->getApaId();
        $line->title = $parserXML->getTitle();
        $line->ndate = $parserXML->getDate();
        $line->content = $parserXML->getText();
        $line->place = $parserXML->getPlace();
        $line->source = $parserXML->getSource();
        $line->keywords = $parserXML->getKeywords();
        $line->author = $parserXML->getAuthor();
        $line->teaser = $parserXML->getTeaser();
        $line->filename = $parserXML->getFilename();
        $line->xml_filename = $xml_filename;

        $relations = $parserXML->getRelations();
        $this->relations = CMap::mergeArray(
            $this->relations,
            array($line->xml_filename => $relations)
        );

        if (isset($line->id)) {
            $childNews = ApalinesNews::model()->findAll('related_maindoc_id=:id', array(':id'=>$line->id));
            foreach ($childNews as $child) {
                if (!in_array($child->xml_filename, $relations)) {
                    $child->delete();
                }
            }
            if (empty($relations)) {
                $line->background_news = 0;
            }
        }

        if ($is_success = $line->save()) {
            if ($isExist) {
                Yii::log("{$line->xml_filename} done  update doc_id:{$line->docid} title:{$line->title} DOK_MELDUNG", 'info');
            } else {
                Yii::log("{$line->xml_filename} done  insert doc_id:{$line->docid} title:{$line->title} DOK_MELDUNG", 'info');
            }
        } else {
            if($isExist){
                Yii::log("{$line->xml_filename} error update doc_id:{$line->docid} title:{$line->title} DOK_MELDUNG", 'info');
            } else {
                Yii::log("{$line->xml_filename} error insert doc_id:{$line->docid} title:{$line->title} DOK_MELDUNG", 'info');
            }
        }
        return $is_success;
    }
    /**
     * Function processes multimedia xml files
     *
     * @param object $parserXML object of parserXml class
     * @param string $xml_filename name of xml file
     *
     * @return boolean whether import finished successfully
     */
    function processBild($parserXML, $xml_filename = null)
    {
        $line = ApalinesMM::model()->find('docid = :docid',
            array(':docid'=>$parserXML->getDocId()));
        $isExist = isset($line);

        if (!isset($line)) {
            $line = new ApalinesMM();
            $line->docid = $parserXML->getDocId();
            $line->channel = $this->name;
        }

        $line->apa_id = $parserXML->getApaId();
        $line->title = $parserXML->getTitle();
        $line->ndate = $parserXML->getDate();
        $line->place = $parserXML->getPlace();
        $line->source = $parserXML->getSource();
        $line->keywords = $parserXML->getKeywords();
        $line->author = $parserXML->getAuthor();
        $line->filename = $parserXML->getFilename();
        $line->xml_filename = $xml_filename;

        $line->type = ($parserXML->getDocType() == 'DOK_BILD')? 0 : 1;

        $files = $parserXML->getFiles();

        if ($is_success = $line->save()) {
            foreach($files as $file) {
                $model = new ApalinesFile();
                $model->attributes = $file;
                switch ($model->size) {
                    case 'ORIGINAL':
                        $model->size = ApalinesFile::ORIGINAL;
                        break;
                    case 'THUMBNAIL':
                        $model->size = ApalinesFile::THUMBNAIL;
                        break;
                    case 'HIRES':
                        $model->size = ApalinesFile::HIRES;
                        break;
                }
                $model->parent_id = $line->id;
                $model->save();
            }

            if ($isExist) {
                Yii::log("{$line->xml_filename} done  update doc_id:{$line->docid} title:{$line->title} BILD", 'info');
            } else {
                Yii::log("{$line->xml_filename} done  insert doc_id:{$line->docid} title:{$line->title} BILD", 'info');
            }
        } else {
            if ($isExist) {
                Yii::log("{$line->xml_filename} error update doc_id:{$line->docid} title:{$line->title} BILD", 'info');
            } else {
                Yii::log("{$line->xml_filename} error insert doc_id:{$line->docid} title:{$line->title} BILD", 'info');
            }
        }
        return $is_success;
    }
    /**
     * Function processes removing xml files
     *
     * @param object $parserXML object of parserXml class
     * @param string $xml_filename name of xml file
     *
     * @return boolean whether import finished successfully
     */
    function processRemoval($xml, $xml_filename = null)
    {
        $parserXML = new parserXML($xml);
        $deleteId = $parserXML->getDeleteId();

        $line = ApalinesNews::model()->find('docid = :docid',array(':docid'=> $deleteId ));
        if (!isset($line)) {
            $line = ApalinesMM::model()->find('docid = :docid',array(':docid'=> $deleteId ));
        }
        if (isset($line)) {
            $line->delete();
            //Moving to done folder
            $this->moveFileToDone($xml_filename);
            Yii::log("{$line->xml_filename} done  delete docid:{$line->docid} title:{$line->title}", 'info');
        } else {
            //Moving to error folder
            $this->moveFileToError($xml_filename);
            Yii::log("{$xml_filename} error delete", 'info');
        }
    }
    /**
     * Function removes news and child news,files
     * those lifetime is up
     *
     */
    function clearOld()
    {
        $module = Yii::app()->getModule('apalines');
        $max_age_of_news = false;
        if (isset($module->channels[$this->name]['max_age_of_news'])) {
            $max_age_of_news = $module->channels[$this->name]['max_age_of_news'];
        }
        if (!$max_age_of_news && isset($module->total_max_age_of_news)) {
            $max_age_of_news = $module->total_max_age_of_news;
        }
        if (!$max_age_of_news) {
            return;
        }

        $news = ApalinesNews::model()->findAll(array(
            'condition' => 'channel = :channel AND deleted <> 1 AND (FROM_UNIXTIME(ndate) < (NOW() - interval :max_age_of_news hour))',
            'params' => array(
                ':channel' => $this->name,
                ':max_age_of_news' => $max_age_of_news
            )
        ));

        foreach ($news as $line) {
            $line->delete();
        }
    }
    /**
     * Function removes multimedia files from public folder by file names
     *
     * @param array $files - names of files which should be removed
     */
    function removeFilesByName($files)
    {
        foreach ($files as $file) {
            if (file_exists($this->makeImagesPath($file))) {
                unlink($this->makeImagesPath($file));
                Yii::log("{$file} done  delete image", 'info');
            } else if(file_exists($this->makeVideosPath($file))) {
                Yii::log("{$file} done  delete video", 'info');
                unlink($this->makeVideosPath($file));
            }
        }
    }
    /**
     * Function removes multimedia records which have not parent news
     */
    function checkMMParents()
    {
        $multimedia = ApalinesMM::model()->findAll('channel = :channel AND deleted=0 AND related_maindoc_id=0', array(':channel'=> $this->name));
        foreach ($multimedia as $mm) {
            $mm->delete();
        }
    }
    /**
     * Function make relations between database apa records
     */
    function makeRelations()
    {
        foreach($this->relations as $parent_xml_filename => $children){

            $parent = ApalinesNews::model()->find('xml_filename = :xml_filename',array(':xml_filename'=>$parent_xml_filename));
            if (isset($parent)) {
                // Make background relations
                foreach ($children as $child_xml_filename) {
                    $child = ApalinesNews::model()->find('xml_filename = :xml_filename', array(':xml_filename'=>$child_xml_filename));
                    if (!isset($child)) {
                        $child = ApalinesMM::model()->find('xml_filename = :xml_filename', array(':xml_filename'=>$child_xml_filename));
                    }
                    if (isset($child)) {
                        // Relation is news-news
                        if ($child instanceof ApalinesNews) {
                            $parent->background_news = 1;
                        }
                        $child->related_maindoc_id = $parent->id;
                        $child->save();
                    }
                }
                $parent->save();
            }
        }
    }
    /**
     * Function makes path to the file which placed in recieved folder path
     *
     * @param string $folder path to folder where the file placed
     * @param string $file name of the file
     *
     * @return string full path to the file
     */
    function makePath($folder,$file)
    {
        return $folder.DIRECTORY_SEPARATOR.$file;
    }
    /**
     * Function makes path to file which placed in work folder
     *
     * @param string $file name of the file
     *
     * @return string full path to the file
     */
    function makeWorkPath($file)
    {
        return $this->makePath($this->workPath, $file);
    }
    /**
     * Function makes path to file which placed in done folder
     *
     * @param string $file name of the file
     *
     * @return string full path to the file
     */
    function makeDonePath($file)
    {
        return $this->makePath($this->donePath, $file);
    }
    /**
     * Function makes path to file which placed in error folder
     *
     * @param string $file name of the file
     *
     * @return string full path to the file
     */
    function makeErrorPath($file)
    {
        return $this->makePath($this->errorPath, $file);
    }
    /**
     * Function makes path to file which placed in images folder
     *
     * @param string $file name of the file
     *
     * @return string full path to the file
     */
    function makeImagesPath($file)
    {
        return $this->makePath($this->imagesPath, $file);
    }
    /**
     * Function makes path to file which placed in videos folder
     *
     * @param string $file name of the file
     *
     * @return string full path to the file
     */
    function makeVideosPath($file)
    {
        return $this->makePath($this->videosPath, $file);
    }
    /**
     * Function moves all files from main channel folder to work folder
     */
    function moveToWork()
    {
        if (is_dir($this->path)) {
            if ($dh = opendir($this->path)) {
                while (($file = readdir($dh)) !== false) {
                    $path = $this->makePath($this->path,$file);
                    $path_parts = pathinfo($path);
                    if(!is_file($path)||$path_parts['extension']=='txt'){
                        continue;
                    }
                    rename($this->makePath($this->path,$file),
                        $this->makeWorkPath($file));
                }
                closedir($dh);
            }
        }
    }
    /**
     * Function moves the file from work folder to done folder
     *
     * @param string $file name of the file
     */
    function moveFileToDone($file)
    {
        rename($this->makeWorkPath($file),
            $this->makeDonePath($file));
    }
    /**
     * Function moves the file from work folder to error folder
     *
     * @param string $file name of the file
     */
    function moveFileToError($file)
    {
        rename($this->makeWorkPath($file),
            $this->makeErrorPath($file));
    }
    /**
     * Function moves the file from done folder to error folder.
     * This function is needed if multimedia was successfully imported but it has not parents,
     * so it should be removed and marked as error
     *
     */
    function moveDoneFileToError($file)
    {
        rename($this->makeDonePath($file),
            $this->makeErrorPath($file));
    }
}
