<?php

class FGForms extends CActiveRecord
{
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return Forms the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'forms';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('json, html', 'required'),
			array('name, user thankyoupage', 'length', 'max'=>255),
			array('salt', 'length', 'max'=>32),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, name, json, html, user, salt, creator_email thankyoupage nextpage prevpage', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'formdata' => array(self::HAS_MANY, 'FGFormdata', 'form_id')
		);
	}
	
	public function defaultScope()
	{
		return array('order'=>'id DESC');
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'name' => 'Name',
			'json' => 'Json',
			'html' => 'Html',
			'user' => 'User',
			'creator_email' => 'Creator Email',
			'salt' => 'Salt',
			'thankyoupage' => 'Thankyoupage',
		);
	}
	
	public function validateInput($inputData = null)
	{
		if($inputData === null)
			$inputData = $_POST;
		$formObject = $this->JSONToArray();
		$validator = new FGValidator();
		foreach($formObject['elements'] as $element)
		{
			if(isset($element['name']))
			{
				$value = isset($_POST[$element['name']])? strval($_POST[$element['name']]) : '';
				if(isset($element['mandatory']) && $element['mandatory'] == true && trim($value) === '')
				{
					$message = Yii::t('yii','{attribute} cannot be blank.');
					$validator->addError($this, $element['name'], $message);
				}
				elseif(isset($element['validation']) && $element['validation'] != '' && $value !== '')
				{
					$validator->validate($element['validation'], $this, $element['name'], $value);
				}
				else
				{
					$validator->validateElement($element['type'], $this, $element['name'], $value);
				}
			}
		}
		return !$this->hasErrors();
	}
	
	public function JSONToArray()
	{
		return CJSON::decode($this->json);
	}
	
	public function getElementFromJSONByName($name)
	{
		$object = $this->JSONToArray();
		foreach($object['elements'] as $element) {
			if(isset($element['name']) && $element['name'] == $name) {
				return $element;
			}
		}
		return false;
	}
	
	public function getElementLabel($name)
	{
		$element = $this->getElementFromJSONByName($name);
		if($element && isset($element['label'])) {
			return $element['label'];
		}
		return $name;
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id,true);
		$criteria->compare('name',$this->name,true);
		$criteria->compare('json',$this->json,true);
		$criteria->compare('html',$this->html,true);
		$criteria->compare('user',$this->user,true);
		$criteria->compare('salt',$this->salt,true);
		// $criteria->compare('thankyoupage',$this->thankyoupage,true);
		// $criteria->compare('nextpage',$this->nextpage,true);
		// $criteria->compare('prevpage',$this->prevpage,true);		

		//if(FGModule::isUserAdmin() == false) {
			// get only user records
			//$criteria->compare('user', Yii::app()->user->id);
		//}
		
		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
			'Pagination'=>array(
				'PageSize'=>40
			)
		));
	}

	/*
		Returns all generated Forms for a user
		@return CActiveDataProvider the data provider that can return the models of all forms
	*/
	public function searchAllForUser($user_email = null)
	{
		$criteria  = new CDbCriteria();
		if($user_email != null){
			$criteria->compare('creator_email', $user_email,true);
		
		}
		return new CActiveDataProvider($this,array(
			'criteria' => $criteria,
			'Pagination' => false
		));
	} 
}
