<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Shoppingcart controller class file.
 */
class ShoppingcartController extends SCBaseController
{
    /**
     * Returns a list of external action classes.
     *
     * @return array
     */


    public function actions(){
        return array(
            'captcha'=>array(
                'class'=>'CCaptchaAction',
                'fixedVerifyCode' => YII_DEBUG ? 'polomo' : null
            ),
        );
    }
    /*
     * The function performs all operations associated with shoppingcart:
     * updates items quantities, saves user profile, shipping and payment information
     * @return void
     */
    public function actionIndex(){

        $itemsCount = count(Order::getOrder()->items);

        if(!$itemsCount || !$this->csrfProtect()){
            $criticalReferrer = Yii::app()->user->getState('criticalReferrer');
            if(isset($criticalReferrer)){
                $this->redirect($criticalReferrer);
            }

            throw new CHttpException(403);
        }

        $step = isset($_REQUEST['step']) ? $_REQUEST['step'] : 1;

        $data = array();

        switch($step){
            case 1:

                    $order_id = Order::getOrder(true);
                    $items = Item::model()->findAll('order_id=:order_id',array(':order_id'=>$order_id));

                    if(isset($_POST['Item']))
                    {
                        $valid=true;
                        foreach($items as $i=>$item)
                        {
                            if(isset($_POST['Item'][$i]) && is_numeric($_POST['Item'][$i]['quantity']) && $_POST['Item'][$i]['quantity'] > 0)
                                $item->quantity = $_POST['Item'][$i]['quantity'];
                            $valid = $item->validate(array('quantity'))
                                                        && is_numeric($_POST['Item'][$i]['quantity'])
                                                        && $_POST['Item'][$i]['quantity'] > 0
                                                        && $valid;
                        }

                        if($valid){
                            foreach($items as $item){
                                $item->save();
                            }

                            Yii::app()->user->setState('step_1', true);
                            $step++;
                        }
                    }

                break;
            case 2:

                    $model = Order::getUser();
                    $this->performAjaxValidation($model);

                    if(isset($_POST['TmpUser']))
                    {
                        $model->attributes=$_POST['TmpUser'];

                        if($model->validate() && $model->unsetExistFields()->save(false)){

                            Yii::app()->user->setState('step_2', true);
                            $step++;
                        }else{
                            $data['user'] = $model;
                        }
                    }
                break;
            case 3:
                    $model = Order::getOrder();

                    if(isset($_POST['Order']))
                    {
                        $model->shipping_info = EncryptHelper::decrypt($_POST['Order']['shipping_info']);
                        $model->payment_info  = EncryptHelper::decrypt($_POST['Order']['payment_info']);

                        if(md5($model->shipping_info.$model->payment_info) == $_POST['checksum']){
                            if($model->save(false)){
                                Yii::app()->user->setState('step_3', true);
                                $step++;
                            }
                        }
                    }
                break;
            case 4:

                $model = Order::getOrder();
                $this->performAjaxValidation($model);

                if(isset($_POST['Order'])){

                    $model->verifyCode = $_POST['Order']['verifyCode'];

                    if(!Yii::app()->user->getState('step_1')){
                        $this->redirect(Yii::app()->createUrl('shoppingcart/shoppingcart'));
                    }else
                    if(!Yii::app()->user->getState('step_2')){
                        $this->redirect(Yii::app()->createUrl('shoppingcart/shoppingcart',array('step'=>2)));
                    }else
                    if(!Yii::app()->user->getState('step_3')){
                        $this->redirect(Yii::app()->createUrl('shoppingcart/shoppingcart',array('step'=>3)));
                    }

                    if($model->validate(array('verifyCode'))){
                        $model->ordered = Order::ORDERED;
                        $this->onOrdered($model);
                        if($model->save()){
                            $items = $model->items;
                            $referrer = $items[count($items)-1]->referrer;

                            Yii::app()->user->setState('step_1',  null);
                            Yii::app()->user->setState('step_2',  null);
                            Yii::app()->user->setState('step_3',  null);
                            Yii::app()->user->setState('user_id', null);

                            $this->redirect($referrer);
                        }
                    }else{
                            $data['order'] = $model;
                    }
                }
                break;
        }

        //Extend the lifespan of cookie
        $cookie = new CHttpCookie('user_id', Order::getUser(true));
        $cookie->expire = time()+60 * Yii::app()->getModule('shoppingcart')->max_lifetime;
        Yii::app()->request->cookies['cookie_name'] = $cookie;
        //

        $data['step'] = $step;
        $this->render('index',array('data'=>$data));
    }
    /*
     * The function performs when a user confirms the order,
     * it marks the order as finished and sends notice to order owner and items administrators
     */
    protected  function onOrdered($order){

        $user = Order::getUser()->implementFromSession();
        $items = $order->items;
        $itemsToAdmins = array();
        foreach($items as $item){
            $itemsToAdmins[$item->admin_email][] = $item;
        }

        $admin_email_template = Yii::app()->getModule('shoppingcart')->adminemail_template;

        foreach($itemsToAdmins as $admin_email => $itemsToAdmin){

            $subject = CommentsModule::t('Items ordered');

            $notification = new Notification(
                $subject,
                $this->renderMail('/mail_templates/'.$admin_email_template, $subject, array('items' => $itemsToAdmin))
            );

            $notification->recipients[] = new EmailRecipient($admin_email);

            Yii::app()->notificationManager->notifyAbout($notification);
        }

        $usermail_template = Yii::app()->getModule('shoppingcart')->usermail_template;

        foreach($items as $item){

            $subject = CommentsModule::t('You ordered items');

            $notification = new Notification(
                $subject,
                $this->renderMail('/mail_templates/'.$usermail_template, $subject, array('items' => $items))
            );

            $notification->recipients[] = new EmailRecipient($user->email);

            Yii::app()->notificationManager->notifyAbout($notification);
        }
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if(isset($_POST['ajax']))
        {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
}