<?php
/**
 * Yii-User module
 *
 * @author Mikhail Mangushev <mishamx@gmail.com>
 * @link http://yii-user.googlecode.com/
 * @license http://www.opensource.org/licenses/bsd-license.php
 * @version $Id: UserModule.php 105 2011-02-16 13:05:56Z mishamx $
 */

class UserModule extends CWebModule
{
    /**
     * @var int
     * @desc items on page
     */
    public $user_page_size = 10;

    /**
     * @var int
     * @desc items on page
     */
    public $fields_page_size = 10;

    /**
     * @var string
     * @desc hash method (md5,sha1 or algo hash function http://www.php.net/manual/en/function.hash.php)
     */
    public $hash = 'md5';

    /**
     * @var boolean
     * @desc use email for activation user account
     */
    public $sendActivationMail = true;

    //Subject of mail in activation mail. site_name - replaced by value 'name' in main.php
    public $activationMailSubject = 'You registered from {site_name}';

    /**
     * @var string email which is displayed in 'Send from:" in activation mail
     */
    public $activationEmail;

    /**
     * @var boolean
     * @desc allow auth for is not active user
     */
    public $loginNotActiv = false;

    /**
     * @var boolean
     * @desc activate user on registration (only $sendActivationMail = false)
     */
    public $activeAfterRegister = false;

    /**
     * Amount of bad login attemps after which you couldnt login for $loginBlockTime
     */
    public $loginBadAttemps = 3;
    
    /**
     * Time in seconds for blocking ability to login after $loginBadAttemps times
     *
     * @public string
     **/
    public $loginBlockTime = 300;

    /**
     * If login widget using Ajax for field validation
     */
    public $loginAjax = false;

    /**
     * @var boolean
     * @desc login after registration (need loginNotActiv or activeAfterRegister = true)
     */
    public $autoLogin = true;

    private $_registrationUrl = array(
        'en' => array("/user/registration")
    );
    private $_recoveryUrl = array(
        'en' => "/user/recovery/recovery"
    );
    private $_logoutUrl = array(
        'en' => "/user/logout"
    );
    private $_profileUrl = array(
        'en' => "/user/profile"
    );
    private $_returnUrl = array(
        'en' => "/user/profile"
    );
    private $_returnLogoutUrl = array(
        'en' => "/user/login"
    );
    private $_noPermissionsUrl = array(
        'en' => '/site/noPermissions'
    );
    private $_loginUrl = array(
        'en' => "/user/login"
    );
    private $_homePageUrl = array(
        'en' => ''
    );
    private $_friendListUrl = array();
    private $_recoveryChangePasswordUrl = array();
    /**
     * @var string
     * @desc link to the CMS page where UserRecoveryWidget is placed. Will be used in userLoginWidget as a href to corresponding page.
     */
    public $fieldsMessage = '';

    /**
     * @var array
     * @desc User model relation from other models
     * @see http://www.yiiframework.com/doc/guide/database.arr
     */
    public $relations = array();

    /**
     * @var array
     * @desc Profile model relation from other models
     */
    public $profileRelations = array();

    /**
     * @var boolean
     */
    public $captcha = array('registration' => true);

    /**
     * @var int number of seconds that the user can remain in logged-in status. Defaults to 0, meaning login till the user closes the browser.
     * If greater than 0, cookie-based login will be used.
     */
    public $loggedInDuration = 0;

    /**
     * @var int minimum length of username field
     */
    public $usernameMin = 4;

    /**
     * @var string SendFrom email for password forgotten mails
     */
    public $passwordForgottenEmail;

    /**
     * @var int minimum length of password field
     */
    public $passwordMin = 4;

    public $userLoginWidgetFlashSuccess = 'UserLoginWidget-success';

    public $userLoginWidgetFlashError = 'UserLoginWidget-error';

    /**
     * @var string if value is not defined admin will not be informed about new user is registered
     */
    public $registrationNotifyAdminEmail = null;
    /**
     * @var string name of the template file which is used to notify administrator
     */
    public $registrationNotifyAdminTemplate = 'notifyAdminAboutRegister';
    /**
     * @var string url to the page where placed UserProfileEditWidget (for approving emails)
     */
    public $profileEditUrl = false;

    /**
     * @var string define which user field will be used in login process. 'email' or 'username'. If field != 'email' then username will be used
     */
    private $_authField = 'email';

    private $_assetsUrl;

    /**
     * Public setter
     *
     * @param string $value value
     *
     * @throws Exception
     * @return void
     */
    public function setAuthField($value)
    {
        if (!in_array($value, array('username', 'email'))) {
            throw new Exception('authField can have value "username" or "email"');
        }
        $this->_authField = $value;
    }

    /**
     * Public getter
     *
     * @return string
     */
    public function getAuthField()
    {
        return $this->_authField;
    }

    /**
     * @param array $value
     */
    public function setLoginUrl(array $value)
    {
        $this->_loginUrl = $value;
    }

    /**
     * @param array $value
     */
    public function setRegistrationUrl(array $value)
    {
        $this->_registrationUrl = $value;
    }

    /**
     * @param array $value
     */
    public function setRecoveryUrl(array $value)
    {
        $this->_recoveryUrl = $value;
    }

      /**
     * @param array $value
     */
    public function setReturnUrl(array $value)
    {
        $this->_returnUrl = $value;
    }

    /**
     * @param array $value
     */
    public function setReturnLogoutUrl(array $value)
    {
        $this->_returnLogoutUrl = $value;
    }

    /**
     * @param array $value
     */
    public function setHomePageUrl(array $value)
    {
        $this->_homePageUrl = $value;
    }

    /**
     * @param array $value
     */
    public function setFriendListUrl(array $value)
    {
        $this->_friendListUrl = $value;
    }

    public function setRecoveryChangePasswordUrl(array $value)
    {
        $this->_recoveryChangePasswordUrl = $value;
    }

    /**
     * @param array $value
     */
    public function setNoPermissionsUrl(array $value)
    {
        $this->_noPermissionsUrl = $value;
    }

    public function getRecoveryChangePasswordUrl($locale = '')
    {
        return $this->_getUrl('_recoveryChangePasswordUrl', $locale);
    }

    public function getLoginUrl($locale = '')
    {
        return $this->_getUrl('_loginUrl', $locale, '/user/login');
    }

    public function getFriendListUrl($locale = '')
    {
        return $this->_getUrl('_friendListUrl', $locale);
    }

    public function getRegistrationUrl($locale = '')
    {
        return $this->_getUrl('_registrationUrl', $locale);
    }

    public function getRecoveryUrl($locale = '')
    {
        return $this->_getUrl('_recoveryUrl', $locale);
    }

    public function getReturnUrl($locale = '')
    {
        return $this->_getUrl('_returnUrl', $locale);
    }

    public function getReturnLogoutUrl($locale = '')
    {
        return $this->_getUrl('_returnLogoutUrl', $locale);
    }

    public function getNoPermissionsUrl($locale = '')
    {
        return $this->_getUrl('_noPermissionsUrl', $locale);
    }

    public function getHomePageUrl($locale = '')
    {
        return $this->_getUrl('_homePageUrl', $locale);
    }

    /**
     * Method for getting right url depends of locale
     *
     * @param string $urlField name of private property
     * @param string $locale   requested locale
     * @param string $default  default value if urls are not set
     *
     * @return mixed
     */
    private function _getUrl($urlField, $locale, $default = '')
    {
        if (empty($locale)) {
            $locale = Yii::app()->language;
        }
        if (isset($this->{$urlField}[$locale])) {
            return $this->{$urlField}[$locale];
        } else {
            $tmp = array_values($this->$urlField);
            return count($this->$urlField) > 0 ? array_shift($tmp) : $default;
        }
    }

    /**
     * @var bool define if username should be unique during registration
     */
    public $usernameUnique = true;

    /**
     * @var bool define if username will be shown in register process
     */
    public $usernameRequired = true;

    /**
     * @var boolean
     */
    //public $cacheEnable = false;

    public $tableUsers = '{{users}}';
    public $tableProfiles = '{{profiles}}';
    public $tableProfileFields = '{{profiles_fields}}';
    public $tableAccounts = '{{accounts}}';

    static private $_user;
    static private $_admin;
    static private $_admins;

    /**
     * @var array
     * @desc Behaviors for models
     */
    public $componentBehaviors = array();

    public function init()
    {
        // this method is called when the module is being created
        // you may place code here to customize the module or the application

        // import the module-level models and components
        $this->setImport(array(
            'user.models.*',
            'user.components.*',
            'user.widgets.*',
            'user.helpers.*',
            'user.services.*',
        ));
        Yii::app()->user->loginUrl = $this->getLoginUrl();
    }

    public function getBehaviorsFor($componentName)
    {
        if (isset($this->componentBehaviors[$componentName])) {
            return $this->componentBehaviors[$componentName];
        } else {
            return array();
        }
    }

    public function beforeControllerAction($controller, $action)
    {
        if (parent::beforeControllerAction($controller, $action)) {
            // this method is called before any module controller action is performed
            // you may place customized code here
            return true;
        } else
            return false;
    }

    /**
     * @param $str
     * @param $params
     * @param $dic
     * @return string
     */
    public static function t($str = '', $params = array(), $dic = 'user')
    {
        return Yii::t("UserModule." . $dic, $str, $params);
    }

    /**
     * @return hash string.
     */
    public static function encrypting($string = "")
    {
        $hash = Yii::app()->getModule('user')->hash;
        if ($hash == "md5")
            return md5($string);
        if ($hash == "sha1")
            return sha1($string);
        else
            return hash($hash, $string);
    }

    /**
     * @param $place
     * @return boolean
     */
    public static function doCaptcha($place = '')
    {
        if (!extension_loaded('gd'))
            return false;
        if (in_array($place, Yii::app()->getModule('user')->captcha))
            return Yii::app()->getModule('user')->captcha[$place];
        return false;
    }

    /**
     * Return admin status.
     * @return boolean
     */
    public static function isAdmin()
    {
        if (Yii::app()->user->isGuest)
            return false;
        else {
            return Yii::app()->user->checkAccess('Admin');
        }
    }

    /**
     * Return safe user data.
     * @param user id not required
     * @return user object or false
     */
    public static function user($id = 0)
    {
        if ($id)
            return User::model()->active()->findbyPk($id);
        else {
            if (Yii::app()->user->isGuest) {
                return false;
            } else {
                if (!self::$_user)
                    self::$_user = User::model()->active()->findbyPk(Yii::app()->user->id);
                return self::$_user;
            }
        }
    }

    public function getAssetsUrl()
    {
        if ($this->_assetsUrl === null) {
            $assetsPath = Yii::getPathOfAlias('user.views.asset');
            $this->_assetsUrl = Yii::app()->getAssetManager()->publish($assetsPath, true);
        }

        return $this->_assetsUrl;
    }

}
