<?php

class RecoveryController extends Controller
{
    public $defaultAction = 'recovery';

    /**
     * Change password
     *
     * This action only handle requests and redirect back. No render
     */
    public function actionChange()
    {
        $form = new UserChangePassword;
        $form->attributes = @$_POST['UserChangePassword'];

        if (isset($_POST['ajax'])) {
            echo CActiveForm::validate($form);
            Yii::app()->end();
        }

        if ($form->validate()) {
            $user = User::model()->notsafe()->findByAttributes(array('email' => @$_POST['email']));
            if (!isset($user) || $user->activkey != @$_POST['activkey']) {
                $form->addError('password', 'Active key is expired');
            }
        }
        if (!$form->hasErrors()) {
            $user->password = Yii::app()->controller->module->encrypting($form->password);
            $user->activkey = Yii::app()->controller->module->encrypting(microtime() . $form->password);
            $user->save();
            Yii::app()->user->setFlash('recoveryMessage', UserModule::t("New password is saved."));
        } else {
            $form->password = null;
            $form->verifyPassword = null;
            Yii::app()->user->setState('UserChangePassword', array('attributes' => $form->attributes, 'errors' => $form->getErrors()));
        }
        if (isset($_GET['return_url'])) {
            $this->redirect($_GET['return_url']);
        }

        $this->redirect(Yii::app()->getModule('user')->recoveryChangePasswordUrl);
    }

    /**
     * This action only handle requests and redirect back. No render
     */
    public function actionChangeRequest()
    {
        $form = new UserRecoveryForm;
        if (Yii::app()->user->isGuest) {
            if (isset($_POST['UserRecoveryForm'])) {
                $form->attributes = $_POST['UserRecoveryForm'];
                if ($form->validate()) {
                    $user = User::model()->with('account')->notsafe()->findbyPk($form->user_id);
                    if ($user->account) {
                        Yii::app()->user->setFlash('recoveryMessage', UserModule::t("You used TPA for logged in. You can not recovery password"));
                    } else {
                        $activation_url = $this->_getChangePasswordUrl(array("activkey" => $user->activkey, "email" => $user->email));
                        $subject = UserModule::t("You have requested the password recovery site {site_name}",
                            array(
                                '{site_name}' => Yii::app()->name,
                            ));
                        $message = $this->renderMail('/mail_templates/recovery', $subject, array('activation_url' => $activation_url));
                        $notification = new Notification(
                            $subject,
                            $message,
                            array(new EmailRecipient($user->email, $this->module->passwordForgottenEmail, '', true))
                        );
                        Yii::app()->notificationManager->notifyAbout($notification);
                        Yii::app()->user->setFlash('recoveryMessage', UserModule::t("Please check your email. An instruction was sent to your email address."));
                    }
                } else {
                    $form->password = null;
                    $form->verifyPassword = null;
                    Yii::app()->user->setState('UserRecoveryForm', array('attributes' => $form->attributes, 'errors' => $form->getErrors()));
                }
            }
        } else {
            Yii::app()->user->setFlash('recoveryMessage', UserModule::t("Please log out."));
        }
        /*  if is set returl_url - we heed to redirect user there   */
        if (isset($_REQUEST['return_url'])) {
            $this->redirect($_REQUEST['return_url']);
        } else {
            $this->render('recovery', array('form' => $form));
        }
    }

    private function _getChangePasswordUrl($params)
    {
        if (preg_match('/.+\.(php|html)$/', $this->module->recoveryChangePasswordUrl)) {
            return $this->createAbsoluteUrl($this->module->recoveryChangePasswordUrl . '?' . http_build_query($params));
        } else {
            return $this->createAbsoluteUrl($this->module->recoveryChangePasswordUrl, $params);
        }
    }

}