<?php
/**
 *
 */
class EmailService extends EAuthServiceBase implements IAuthService, IUserFields
{
    /**
     * Everything is OK.
     */
    const ERROR_NONE = 0;
    /**
     * Failed to authenticate authentication identity (email-password mismatch).
     */
    const ERROR_AUTH_FAILED = 1;
    /**
     * No errors occured. Activation mail to user was sent and user will be registered
     * as soon as he will click the activation url in the email message body.
     */
    const ERROR_ACTIVATION_MAIL_SENT = 2;
    /**
     * Error occured while trying to send activation mail.
     */
    const ERROR_ACTIVATION_MAIL_FAILED = 3;

    protected $name = 'email';
    protected $title = 'E-mail';
    protected $type = 'email';



    /**
     * @var string email field, put here email address, you want to authenticate
     */
    public $username;
    /**
     * @var string password field, put here password, with which you want to authenticate
     */
    public $password;
    /**
     * @var string email field
     */
    public $email;

    /**
     * @var integer code of the error, that occured, ERROR_NONE will be set if no errors occured
     */
    public $errorCode;

    /**
     * @var LUser user-owner of the email account, specified on bind process
     * (null means that we're trying to authenticate, not bind an account to already existing user)
     */
    public $user = null;

    /**
     * This function simply tries to authenticate auth identity (see docs about user authenticating behaviour)
     * @return bool is user identity authenticated or authentication failed
     */
    public function authenticate()
    {
        $email = $this->email;
        $username = $this->username;
        $password = $this->password;
        $password_hash = Yii::app()->getModule('user')->encrypting($password);
        $this->authenticated = false;

        if ($this->authField == 'email') {
            $user = User::model()->findByAttributes(array('email' => $email, 'password' => $password_hash));
        } else if ($this->authField == 'username') {
            $user = User::model()->findByAttributes(array('username' => $username, 'password' => $password_hash));
        } else
            return false;

        if ($user) {
            $this->attributes['id'] = $this->attributes['email'] = $this->attributes['displayId'] = $username;
            $this->authenticated = true;
            $this->errorCode = self::ERROR_NONE;
        } else {
            $this->errorCode = self::ERROR_AUTH_FAILED;
        }
        Yii::log("EmailService auth resulted with code $this->errorCode.", CLogger::LEVEL_INFO);
        return $this->authenticated;
    }

    /**
     * Simply redirects user to the specified url (if not specified, redirectUrl property will be used instead)
     *
     * We have to override this function, because email authentication doesn't require popup.
     * @param string $url
     */
    public function redirect($url = null)
    {
        Yii::app()->request->redirect(isset($url) ? $url : $this->getRedirectUrl(), true);
    }

    /**
     * Simply redirects user to the specified url (if not specified, cancelUrl property will be used instead)
     *
     * We have to override this function, because email authentication doesn't require popup.
     * @param string $url
     */
    public function cancel($url = null)
    {
        Yii::app()->request->redirect(isset($url) ? $url : $this->getCancelUrl(), false);
    }

    public function getEmail()
    {

    }

    public function getName()
    {
        return $this->username;
    }

    public function getError()
    {
        return UserModule::t('Not correct username or password');
    }
}
